
/*
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
 * OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef _PYPGMODULE_H
#define _PYPGMODULE_H

extern PyObject* Error;
extern PyObject* Warning;
extern PyObject* InterfaceError;
extern PyObject* DatabaseError;
extern PyObject* InternalError;
extern PyObject* OperationalError;
extern PyObject* ProgrammingError;
extern PyObject* IntegrityError;
extern PyObject* DataError;
extern PyObject* NotSupportedError;

/*
 * Sets and exception based on the ODBC SQLSTATE and error message and returns zero.  If either handle is not
 * available, pass SQL_NULL_HANDLE.
 */
PyObject* RaiseErrorFromHandle(HDBC hdbc, HSTMT hstmt);

/*
 * Sets an exception and returns zero.
 *
 * szSqlState: The optional SQLSTATE reported by ODBC.  If not provided (sqlstate is NULL or sqlstate[0] is NULL),
 * "HY000" (General Error) is used.  Note that HY000 causes Error to be used if exc_class is not provided.
 *
 * exc_class: The exception class (DatabaseError, etc.) to construct.  If NULL, the appropriate class will be
 * determined from the SQLSTATE.
 */
PyObject* RaiseErrorV(const char* sqlstate, PyObject* exc_class, const char* format, ...);

// Type objects such as 'int()'.

extern PyObject* str_type;
extern PyObject* float_type;
extern PyObject* int_type;
extern PyObject* buffer_type;
extern PyObject* bool_type;
extern PyObject* long_type;
extern PyObject* decimal_type;

inline bool PyDecimal_Check(PyObject* p)
{
    return p->ob_type == (_typeobject*)decimal_type;
}

extern HENV henv;

extern PyTypeObject RowType;
extern PyTypeObject CursorType;
extern PyTypeObject ConnectionType;

// Thd pyodbc module.
extern PyObject* pModule;

inline bool lowercase()
{
    return PyObject_GetAttrString(pModule, "lowercase") == Py_True;
}

extern char chDecimal;
extern char chGroupSeparator;
extern char chCurrencySymbol;


#endif // _PYPGMODULE_H
